@@ -31,17 +31,23 @@
 #' @param .shape Must be strictly positive.
 #' @param .dispersion An alternative way to specify the `.shape`.
 #' @param .num_sims The number of randomly generated simulations you want.
+#' @param .return_tibble A logical value indicating whether to return the result
+#' as a tibble. Default is TRUE.
 #'
 #' @examples
 #' tidy_inverse_normal()
+#'
 #' @return
 #' A tibble of randomly generated data.
 #'
+#' @name tidy_inverse_normal
+NULL
+
 #' @export
-#'
+#' @rdname tidy_inverse_normal
 
 tidy_inverse_normal <- function(.n = 50, .mean = 1, .shape = 1, .dispersion = 1 / .shape,
-                                .num_sims = 1) {
+                                .num_sims = 1, .return_tibble = TRUE) {
 
   # Tidyeval ----
   n <- as.integer(.n)
@@ -49,6 +55,7 @@ tidy_inverse_normal <- function(.n = 50, .mean = 1, .shape = 1, .dispersion = 1
   mu <- as.numeric(.mean)
   shape <- as.numeric(.shape)
   dispers <- as.numeric(.dispersion)
+  ret_tbl <- as.logical(.return_tibble)
 
   # Checks ----
   if (!is.integer(n) | n < 0) {
@@ -85,27 +92,28 @@ tidy_inverse_normal <- function(.n = 50, .mean = 1, .shape = 1, .dispersion = 1
   qs <- seq(0, 1, (1 / (n - 1)))
   ps <- qs
 
-  df <- dplyr::tibble(sim_number = as.factor(x)) %>%
-    dplyr::group_by(sim_number) %>%
-    dplyr::mutate(x = list(1:n)) %>%
-    dplyr::mutate(y = list(actuar::rinvgauss(n,
-      mean = mu, shape = shape,
-      dispersion = dispers
-    ))) %>%
-    dplyr::mutate(d = list(density(unlist(y), n = n)[c("x", "y")] %>%
-      purrr::set_names("dx", "dy") %>%
-      dplyr::as_tibble())) %>%
-    dplyr::mutate(p = list(actuar::pinvgauss(unlist(y),
-      mean = mu, shape = shape,
-      dispersion = dispers
-    ))) %>%
-    dplyr::mutate(q = list(actuar::qinvgauss(unlist(p),
-      mean = mu, shape = shape,
-      dispersion = dispers
-    ))) %>%
-    tidyr::unnest(cols = c(x, y, d, p, q)) %>%
-    dplyr::ungroup()
+  # Create a data.table with one row per simulation
+  df <- data.table::CJ(sim_number = factor(1:num_sims), x = 1:n)
 
+  # Group the data by sim_number and add columns for x and y
+  df[, y := actuar::rinvgauss(n = .N, mean = mu, shape = shape, dispersion = dispers)]
+
+  # Compute the density of the y values and add columns for dx and dy
+  df[, c("dx", "dy") := density(y, n = n)[c("x", "y")], by = sim_number]
+
+  # Compute the p-values for the y values and add a column for p
+  df[, p := actuar::pinvgauss(y, mean = mu, shape = shape, dispersion = dispers)]
+
+  # Compute the q-values for the p-values and add a column for q
+  df[, q := actuar::qinvgauss(p, mean = mu, shape = shape, dispersion = dispers)]
+
+  if(.return_tibble){
+    df <- dplyr::as_tibble(df)
+  } else {
+    data.table::setkey(df, NULL)
+  }
+
+  # Create a tibble with the parameter grid
   param_grid <- dplyr::tibble(.mean, .shape, .dispersion)
 
   # Attach descriptive attributes to tibble
@@ -115,6 +123,7 @@ tidy_inverse_normal <- function(.n = 50, .mean = 1, .shape = 1, .dispersion = 1
   attr(df, ".dispersion") <- .dispersion
   attr(df, ".n") <- .n
   attr(df, ".num_sims") <- .num_sims
+  attr(df, ".ret_tbl") <- .return_tibble
   attr(df, "tibble_type") <- "tidy_inverse_gaussian"
   attr(df, "ps") <- ps
   attr(df, "qs") <- qs
